@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -50,7 +50,6 @@ escape_string (const char *string)
 {
 	const char *start, *inptr;
 	GString *str;
-	char *buf;
 	
 	str = g_string_new ("");
 	
@@ -68,32 +67,41 @@ escape_string (const char *string)
 		}
 	}
 	
-	buf = str->str;
-	g_string_free (str, FALSE);
-	
-	return buf;
+	return g_string_free (str, FALSE);
 }
 
 static void
 write_part_bodystructure (GMimeObject *part, FILE *fp)
 {
+	GMimeContentType *content_type;
+	GMimeParamList *params;
+	const char *subtype;
 	GMimeParam *param;
+	int i, n;
 	
 	fputc ('(', fp);
 	
-	fprintf (fp, "\"%s\" ", part->content_type->type);
-	if (part->content_type->subtype)
-		fprintf (fp, "\"%s\" ", part->content_type->subtype);
+	content_type = g_mime_object_get_content_type (part);
+	
+	fprintf (fp, "\"%s\" ", g_mime_content_type_get_media_type (content_type));
+	
+	if ((subtype = g_mime_content_type_get_media_subtype (content_type)))
+		fprintf (fp, "\"%s\" ", subtype);
 	else
 		fputs ("\"\"", fp);
 	
 	/* Content-Type params */
-	if ((param = part->content_type->params)) {
+	params = g_mime_content_type_get_parameters (content_type);
+	if ((n = g_mime_param_list_length (params)) > 0) {
 		fputc ('(', fp);
-		while (param) {
-			fprintf (fp, "\"%s\" \"%s\"", param->name, param->value);
-			if ((param = param->next))
+		for (i = 0; i < n; i++) {
+			if (i > 0)
 				fputc (' ', fp);
+			
+			param = g_mime_param_list_get_parameter_at (params, i);
+			fprintf (fp, "\"%s\" \"%s\"", g_mime_param_get_name (param),
+				 g_mime_param_get_value (param));
+				
 		}
 		fputs (") ", fp);
 	} else {
@@ -103,7 +111,6 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 	if (GMIME_IS_MULTIPART (part)) {
 		GMimeMultipart *multipart = (GMimeMultipart *) part;
 		GMimeObject *subpart;
-		int i, n;
 		
 		n = g_mime_multipart_get_count (multipart);
 		for (i = 0; i < n; i++) {
@@ -120,9 +127,8 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 		/* print envelope */
 		fputc ('(', fp);
 		
-		nstring = g_mime_message_get_date_as_string (message);
-		fprintf (fp, "\"%s\" ", nstring);
-		g_free (nstring);
+		str = g_mime_object_get_header ((GMimeObject *) message, "Date");
+		fprintf (fp, "\"%s\" ", str);
 		
 		if ((str = g_mime_object_get_header ((GMimeObject *) message, "Subject")))
 			nstring = escape_string (str);
@@ -192,14 +198,23 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 		/* print body */
 		write_part_bodystructure ((GMimeObject *) message->mime_part, fp);
 	} else if (GMIME_IS_PART (part)) {
-		if (GMIME_OBJECT (part)->disposition) {
-			fprintf (fp, "\"%s\" ", GMIME_OBJECT (part)->disposition->disposition);
-			if ((param = GMIME_OBJECT (part)->disposition->params)) {
+		GMimeContentDisposition *disposition;
+		
+		disposition = g_mime_object_get_content_disposition (part);
+		
+		if (disposition) {
+			fprintf (fp, "\"%s\" ", g_mime_content_disposition_get_disposition (disposition));
+			
+			params = g_mime_content_disposition_get_parameters (disposition);
+			if ((n = g_mime_param_list_length (params)) > 0) {
 				fputc ('(', fp);
-				while (param) {
-					fprintf (fp, "\"%s\" \"%s\"", param->name, param->value);
-					if ((param = param->next))
+				for (i = 0; i < n; i++) {
+					if (i > 0)
 						fputc (' ', fp);
+					
+					param = g_mime_param_list_get_parameter_at (params, i);
+					fprintf (fp, "\"%s\" \"%s\"", g_mime_param_get_name (param),
+						 g_mime_param_get_value (param));
 				}
 				fputs (") ", fp);
 			} else {
@@ -209,7 +224,7 @@ write_part_bodystructure (GMimeObject *part, FILE *fp)
 			fputs ("NIL NIL ", fp);
 		}
 		
-		switch (GMIME_PART (part)->encoding) {
+		switch (g_mime_part_get_content_encoding ((GMimePart *) part)) {
 		case GMIME_CONTENT_ENCODING_7BIT:
 			fputs ("\"7bit\"", fp);
 			break;
@@ -254,6 +269,7 @@ write_bodystructure (GMimeMessage *message, const char *uid)
 static void
 write_header (GMimeMessage *message, const char *uid)
 {
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
 	char *buf;
 	FILE *fp;
 	
@@ -261,7 +277,7 @@ write_header (GMimeMessage *message, const char *uid)
 	fp = fopen (buf, "wt");
 	g_free (buf);
 	
-	buf = g_mime_object_get_headers ((GMimeObject *) message);
+	buf = g_mime_object_get_headers ((GMimeObject *) message, format);
 	
 	fwrite (buf, 1, strlen (buf), fp);
 	g_free (buf);
@@ -272,8 +288,10 @@ write_header (GMimeMessage *message, const char *uid)
 static void
 write_part (GMimeObject *part, const char *uid, const char *spec)
 {
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
 	GMimeStream *istream, *ostream;
 	GMimeMultipart *multipart;
+	GMimeDataWrapper *content;
 	GMimeMessage *message;
 	GMimeObject *subpart;
 	char *buf, *id;
@@ -284,7 +302,7 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 	fp = fopen (buf, "wt");
 	g_free (buf);
 	
-	buf = g_mime_object_get_headers (part);
+	buf = g_mime_object_get_headers (part, format);
 	fwrite (buf, 1, strlen (buf), fp);
 	g_free (buf);
 	
@@ -307,10 +325,10 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 		fp = fopen (buf, "wt");
 		g_free (buf);
 		
-		message = GMIME_MESSAGE_PART (part)->message;
+		message = g_mime_message_part_get_message ((GMimeMessagePart *) part);
 		
 		ostream = g_mime_stream_file_new (fp);
-		g_mime_object_write_to_stream (GMIME_OBJECT (message), ostream);
+		g_mime_object_write_to_stream ((GMimeObject *) message, format, ostream);
 		g_object_unref (ostream);
 	} else if (GMIME_IS_PART (part)) {
 		buf = g_strdup_printf ("%s/%s.TEXT", uid, spec);
@@ -318,7 +336,8 @@ write_part (GMimeObject *part, const char *uid, const char *spec)
 		g_free (buf);
 		
 		ostream = g_mime_stream_file_new (fp);
-		istream = g_mime_data_wrapper_get_stream (GMIME_PART (part)->content);
+		content = g_mime_part_get_content ((GMimePart *) part);
+		istream = g_mime_data_wrapper_get_stream (content);
 		g_mime_stream_write_to_stream (istream, ostream);
 		g_object_unref (ostream);
 	}
@@ -351,11 +370,11 @@ struct _bodystruct {
 	struct {
 		char *type;
 		char *subtype;
-		GMimeParam *params;
+		GMimeParamList *params;
 	} content;
 	struct {
 		char *type;
-		GMimeParam *params;
+		GMimeParamList *params;
 	} disposition;
 	char *encoding;
 	struct _envelope *envelope;
@@ -415,49 +434,53 @@ decode_qstring (unsigned char **in, unsigned char *inend)
 	return qstring;
 }
 
-static GMimeParam *
-decode_param (unsigned char **in, unsigned char *inend)
+static gboolean
+decode_param (unsigned char **in, unsigned char *inend, char **name, char **value)
 {
 	GMimeParam *param;
-	char *name, *val;
+	char *n, *v;
 	
-	if (!(name = decode_qstring (in, inend)))
-		return NULL;
+	if (!(n = decode_qstring (in, inend)))
+		return FALSE;
 	
-	g_assert ((val = decode_qstring (in, inend)));
+	if (!(v = decode_qstring (in, inend))) {
+		g_free (n);
+		return FALSE;
+	}
 	
-	param = g_mime_param_new (name, val);
-	g_free (name);
-	g_free (val);
+	*name = n;
+	*value = v;
 	
-	return param;
+	return TRUE;
 }
 
-static GMimeParam *
+static GMimeParamList *
 decode_params (unsigned char **in, unsigned char *inend)
 {
-	GMimeParam *params, *tail, *n;
+	GMimeParamList *params;
 	unsigned char *inptr;
+	char *name, *value;
 	
 	inptr = *in;
-	params = NULL;
-	tail = (GMimeParam *) &params;
+	
+	params = g_mime_param_list_new ();
 	
 	while (inptr < inend && *inptr == ' ')
 		inptr++;
 	
 	if (inptr == inend) {
 		g_assert_not_reached ();
-		return NULL;
+		return params;
 	}
 	
 	if (strncmp ((const char *) inptr, "NIL", 3) != 0) {
 		g_assert (*inptr == '(');
 		inptr++;
 		
-		while ((n = decode_param (&inptr, inend)) != NULL) {
-			tail->next = n;
-			tail = n;
+		while (decode_param (&inptr, inend, &name, &value)) {
+			g_mime_param_list_set_parameter (params, name, value);
+			g_free (value);
+			g_free (name);
 			
 			while (inptr < inend && *inptr == ' ')
 				inptr++;
@@ -597,7 +620,7 @@ static void
 bodystruct_dump (struct _bodystruct *part, int depth)
 {
 	GMimeParam *param;
-	int i;
+	int i, n;
 	
 	for (i = 0; i < depth; i++)
 		fputs ("  ", stderr);
@@ -606,10 +629,11 @@ bodystruct_dump (struct _bodystruct *part, int depth)
 		 part->content.subtype);
 	
 	if (part->content.params) {
-		param = part->content.params;
-		while (param) {
-			fprintf (stderr, "; %s=%s", param->name, param->value);
-			param = param->next;
+		n = g_mime_param_list_length (part->content.params);
+		for (i = 0; i < n; i++) {
+			param = g_mime_param_list_get_parameter_at (part->content.params, i);
+			fprintf (stderr, "; %s=%s", g_mime_param_get_name (param),
+				 g_mime_param_get_value (param));
 		}
 	}
 	
@@ -662,10 +686,11 @@ bodystruct_dump (struct _bodystruct *part, int depth)
 				fputs ("  ", stderr);
 			fprintf (stderr, "Content-Disposition: %s", part->disposition.type);
 			if (part->disposition.params) {
-				param = part->disposition.params;
-				while (param) {
-					fprintf (stderr, "; %s=%s", param->name, param->value);
-					param = param->next;
+				n = g_mime_param_list_length (part->disposition.params);
+				for (i = 0; i < n; i++) {
+					param = g_mime_param_list_get_parameter_at (part->disposition.params, i);
+					fprintf (stderr, "; %s=%s", g_mime_param_get_name (param),
+						 g_mime_param_get_value (param));
 				}
 			}
 			
@@ -691,11 +716,11 @@ bodystruct_free (struct _bodystruct *node)
 		g_free (node->content.type);
 		g_free (node->content.subtype);
 		if (node->content.params)
-			g_mime_param_destroy (node->content.params);
+			g_object_unref (node->content.params);
 		
 		g_free (node->disposition.type);
 		if (node->disposition.params)
-			g_mime_param_destroy (node->disposition.params);
+			g_object_unref (node->disposition.params);
 		
 		g_free (node->encoding);
 		
@@ -743,7 +768,7 @@ reconstruct_part_content (GMimePart *part, const char *uid, const char *spec)
 	content = g_mime_data_wrapper_new_with_stream (stream, part->encoding);
 	g_object_unref (stream);
 	
-	g_mime_part_set_content_object (part, content);
+	g_mime_part_set_content (part, content);
 }
 
 static void
@@ -756,7 +781,7 @@ reconstruct_message_part (GMimeMessagePart *msgpart, const char *uid, const char
 	int fd;
 	
 	filename = g_strdup_printf ("%s/%s.TEXT", uid, spec);
-	if ((fd = open (filename, O_RDONLY, 0))) {
+	if ((fd = open (filename, O_RDONLY, 0)) == -1) {
 		g_mime_message_part_set_message (msgpart, NULL);
 		g_free (filename);
 		return;
@@ -766,10 +791,9 @@ reconstruct_message_part (GMimeMessagePart *msgpart, const char *uid, const char
 	
 	stream = g_mime_stream_fs_new (fd);
 	parser = g_mime_parser_new_with_stream (stream);
-	g_mime_parser_set_scan_from (parser, FALSE);
 	g_object_unref (stream);
 	
-	message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser, NULL);
 	g_object_unref (parser);
 	
 	g_mime_message_part_set_message (msgpart, message);
@@ -821,10 +845,9 @@ reconstruct_multipart (GMimeMultipart *multipart, struct _bodystruct *body,
 		
 		stream = g_mime_stream_fs_new (fd);
 		parser = g_mime_parser_new_with_stream (stream);
-		g_mime_parser_set_scan_from (parser, FALSE);
 		g_object_unref (stream);
 		
-		subpart = g_mime_parser_construct_part (parser);
+		subpart = g_mime_parser_construct_part (parser, NULL);
 		g_object_unref (parser);
 		
 		if (GMIME_IS_MULTIPART (subpart)) {
@@ -845,6 +868,8 @@ reconstruct_multipart (GMimeMultipart *multipart, struct _bodystruct *body,
 static void
 reconstruct_message (const char *uid)
 {
+	GMimeFormatOptions *format = g_mime_format_options_get_default ();
+	GMimeObject *mime_part;
 	GMimeMessage *message;
 	GMimeParser *parser;
 	GMimeStream *stream;
@@ -861,15 +886,16 @@ reconstruct_message (const char *uid)
 	
 	stream = g_mime_stream_fs_new (fd);
 	parser = g_mime_parser_new_with_stream (stream);
-	g_mime_parser_set_scan_from (parser, FALSE);
 	g_object_unref (stream);
 	
 	/* constructs message object and toplevel mime part (although
 	   the toplevel mime part will not have any content... */
-	message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser, NULL);
 	g_object_unref (parser);
 	
-	if (GMIME_IS_MULTIPART (message->mime_part)) {
+	mime_part = g_mime_message_get_mime_part (message);
+	
+	if (GMIME_IS_MULTIPART (mime_part)) {
 		struct _bodystruct *body;
 		GByteArray *buffer;
 		GMimeStream *mem;
@@ -888,16 +914,16 @@ reconstruct_message (const char *uid)
 		g_mime_stream_write_to_stream (stream, mem);
 		g_object_unref (stream);
 		
-		buffer = GMIME_STREAM_MEM (mem)->buffer;
+		buffer = g_mime_stream_mem_get_byte_array ((GMimeStreamMem *) mem);
 		body = bodystruct_parse (buffer->data, buffer->len);
 		g_object_unref (mem);
 		
 		bodystruct_dump (body, 0);
 		
-		reconstruct_multipart ((GMimeMultipart *) message->mime_part, body, uid, "1");
+		reconstruct_multipart ((GMimeMultipart *) mime_part, body, uid, "1");
 		bodystruct_free (body);
-	} else if (GMIME_IS_PART (message->mime_part)) {
-		reconstruct_part_content ((GMimePart *) message->mime_part, uid, "1");
+	} else if (GMIME_IS_PART (mime_part)) {
+		reconstruct_part_content ((GMimePart *) mime_part, uid, "1");
 	}
 	
 	filename = g_strdup_printf ("%s/MESSAGE", uid);
@@ -908,7 +934,7 @@ reconstruct_message (const char *uid)
 	
 	g_free (filename);
 	stream = g_mime_stream_fs_new (fd);
-	g_mime_object_write_to_stream ((GMimeObject *) message, stream);
+	g_mime_object_write_to_stream ((GMimeObject *) message, format, stream);
 	g_object_unref (message);
 	g_object_unref (stream);
 }
@@ -916,7 +942,7 @@ reconstruct_message (const char *uid)
 
 int main (int argc, char **argv)
 {
-	gboolean scan_from = FALSE;
+	GMimeFormat format = GMIME_FORMAT_MESSAGE;
 	GMimeMessage *message;
 	GMimeParser *parser;
 	GMimeStream *stream;
@@ -926,10 +952,10 @@ int main (int argc, char **argv)
 	if (argc < 2)
 		return 0;
 	
-	g_mime_init (0);
+	g_mime_init ();
 	
-	if (!strcmp (argv[i], "-f")) {
-		scan_from = TRUE;
+	if (!strcmp (argv[i], "--mbox")) {
+		format = GMIME_FORMAT_MBOX;
 		i++;
 	}
 	
@@ -939,10 +965,10 @@ int main (int argc, char **argv)
 	stream = g_mime_stream_fs_new (fd);
 	
 	parser = g_mime_parser_new_with_stream (stream);
-	g_mime_parser_set_scan_from (parser, scan_from);
+	g_mime_parser_set_format (parser, format);
 	g_object_unref (stream);
 	
-	message = g_mime_parser_construct_message (parser);
+	message = g_mime_parser_construct_message (parser, NULL);
 	g_object_unref (parser);
 	
 	if (message) {
