@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -602,10 +602,10 @@ filter_reset (GMimeFilter *filter)
 GMimeFilter *
 g_mime_filter_enriched_new (guint32 flags)
 {
-	GMimeFilterEnriched *new;
+	GMimeFilterEnriched *enriched;
 	
-	new = g_object_newv (GMIME_TYPE_FILTER_ENRICHED, 0, NULL);
-	new->flags = flags;
+	enriched = g_object_new (GMIME_TYPE_FILTER_ENRICHED, NULL);
+	enriched->flags = flags;
 	
-	return (GMimeFilter *) new;
+	return (GMimeFilter *) enriched;
 }
