@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -45,46 +45,48 @@ typedef struct _GMimeMessageClass GMimeMessageClass;
 
 
 /**
- * GMimeRecipientType:
- * @GMIME_RECIPIENT_TYPE_TO: Represents the recipients in the To: header.
- * @GMIME_RECIPIENT_TYPE_CC: Represents the recipients in the Cc: header.
- * @GMIME_RECIPIENT_TYPE_BCC: Represents the recipients in the Bcc: header.
+ * GMimeAddressType:
+ * @GMIME_ADDRESS_TYPE_SENDER: Represents the addresses in the Sender header.
+ * @GMIME_ADDRESS_TYPE_FROM: Represents the addresses in the From header.
+ * @GMIME_ADDRESS_TYPE_REPLY_TO: Represents the addresses in the Reply-To header.
+ * @GMIME_ADDRESS_TYPE_TO: Represents the recipients in the To header.
+ * @GMIME_ADDRESS_TYPE_CC: Represents the recipients in the Cc header.
+ * @GMIME_ADDRESS_TYPE_BCC: Represents the recipients in the Bcc header.
  *
- * A message recipient type.
+ * An address type.
  **/
-typedef enum _GMimeRecipientType {
-	GMIME_RECIPIENT_TYPE_TO,
-	GMIME_RECIPIENT_TYPE_CC,
-	GMIME_RECIPIENT_TYPE_BCC
-} GMimeRecipientType;
+typedef enum _GMimeAddressType {
+	GMIME_ADDRESS_TYPE_SENDER,
+	GMIME_ADDRESS_TYPE_FROM,
+	GMIME_ADDRESS_TYPE_REPLY_TO,
+	GMIME_ADDRESS_TYPE_TO,
+	GMIME_ADDRESS_TYPE_CC,
+	GMIME_ADDRESS_TYPE_BCC
+} GMimeAddressType;
 
 
 /**
  * GMimeMessage:
  * @parent_object: parent #GMimeObject
+ * @addrlists: a table of address lists
  * @mime_part: toplevel MIME part
- * @recipients: hash table of recipients using recipient header name as the hash key
  * @message_id: Message-Id string
- * @reply_to: Reply-To string
- * @subject: Subject string
- * @from: From string
  * @date: Date value
- * @tz_offset: timezone offset
+ * @subject: Subject string
  *
  * A MIME Message object.
  **/
 struct _GMimeMessage {
 	GMimeObject parent_object;
 	
-	InternetAddressList **recipients;
+	InternetAddressList **addrlists;
 	GMimeObject *mime_part;
 	char *message_id;
-	char *reply_to;
+	GDateTime *date;
 	char *subject;
-	char *from;
 	
-	time_t date;
-	int tz_offset;
+	/* < private > */
+	char *marker;
 };
 
 struct _GMimeMessageClass {
@@ -97,23 +99,22 @@ GType g_mime_message_get_type (void);
 
 GMimeMessage *g_mime_message_new (gboolean pretty_headers);
 
-void g_mime_message_set_sender (GMimeMessage *message, const char *sender);
-const char *g_mime_message_get_sender (GMimeMessage *message);
+InternetAddressList *g_mime_message_get_from (GMimeMessage *message);
+InternetAddressList *g_mime_message_get_sender (GMimeMessage *message);
+InternetAddressList *g_mime_message_get_reply_to (GMimeMessage *message);
+InternetAddressList *g_mime_message_get_to (GMimeMessage *message);
+InternetAddressList *g_mime_message_get_cc (GMimeMessage *message);
+InternetAddressList *g_mime_message_get_bcc (GMimeMessage *message);
 
-void g_mime_message_set_reply_to (GMimeMessage *message, const char *reply_to);
-const char *g_mime_message_get_reply_to (GMimeMessage *message);
-
-void g_mime_message_add_recipient (GMimeMessage *message, GMimeRecipientType type, const char *name, const char *addr);
-InternetAddressList *g_mime_message_get_recipients (GMimeMessage *message, GMimeRecipientType type);
+void g_mime_message_add_mailbox (GMimeMessage *message, GMimeAddressType type, const char *name, const char *addr);
+InternetAddressList *g_mime_message_get_addresses (GMimeMessage *message, GMimeAddressType type);
 InternetAddressList *g_mime_message_get_all_recipients (GMimeMessage *message);
 
-void g_mime_message_set_subject (GMimeMessage *message, const char *subject);
+void g_mime_message_set_subject (GMimeMessage *message, const char *subject, const char *charset);
 const char *g_mime_message_get_subject (GMimeMessage *message);
 
-void g_mime_message_set_date (GMimeMessage *message, time_t date, int tz_offset);
-void g_mime_message_get_date (GMimeMessage *message, time_t *date, int *tz_offset);
-void g_mime_message_set_date_as_string (GMimeMessage *message, const char *str);
-char *g_mime_message_get_date_as_string (GMimeMessage *message);
+void g_mime_message_set_date (GMimeMessage *message, GDateTime *date);
+GDateTime *g_mime_message_get_date (GMimeMessage *message);
 
 void g_mime_message_set_message_id (GMimeMessage *message, const char *message_id);
 const char *g_mime_message_get_message_id (GMimeMessage *message);
