@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -23,16 +23,11 @@
 #include <config.h>
 #endif
 
-#include <stdlib.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <string.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <time.h>
 
 #include "gmime-multipart.h"
+#include "gmime-internal.h"
+#include "gmime-common.h"
 #include "gmime-utils.h"
 
 
@@ -48,14 +43,14 @@
  * A #GMimeMultipart represents all multipart MIME container parts.
  **/
 
-
 /* GObject class methods */
 static void g_mime_multipart_class_init (GMimeMultipartClass *klass);
 static void g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClass *klass);
 static void g_mime_multipart_finalize (GObject *object);
 
 /* GMimeObject class methods */
-static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static ssize_t multipart_write_to_stream (GMimeObject *object, GMimeFormatOptions *options,
+					  gboolean content_only, GMimeStream *stream);
 static void multipart_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
 /* GMimeMultipart class methods */
@@ -130,6 +125,7 @@ static void
 g_mime_multipart_init (GMimeMultipart *multipart, GMimeMultipartClass *klass)
 {
 	multipart->children = g_ptr_array_new ();
+	multipart->write_end_boundary = TRUE;
 	multipart->preface = NULL;
 	multipart->postface = NULL;
 }
@@ -152,39 +148,39 @@ g_mime_multipart_finalize (GObject *object)
 }
 
 static ssize_t
-multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
+multipart_write_to_stream (GMimeObject *object, GMimeFormatOptions *options, gboolean content_only, GMimeStream *stream)
 {
 	GMimeMultipart *multipart = (GMimeMultipart *) object;
+	const char *boundary, *newline;
 	ssize_t nwritten, total = 0;
-	const char *boundary;
 	GMimeObject *part;
 	guint i;
 	
-	/* make sure a boundary is set unless we are writing out a raw
-	 * header (in which case it should already be set... or if
-	 * not, then it's a broken multipart and so we don't want to
-	 * alter it or we'll completely break the output) */
 	boundary = g_mime_object_get_content_type_parameter (object, "boundary");
-	if (!boundary && !g_mime_header_list_get_stream (object->headers)) {
-		g_mime_multipart_set_boundary (multipart, NULL);
-		boundary = g_mime_object_get_content_type_parameter (object, "boundary");
+	newline = g_mime_format_options_get_newline (options);
+	
+	if (!content_only) {
+		/* write the content headers */
+		if ((nwritten = g_mime_header_list_write_to_stream (object->headers, options, stream)) == -1)
+			return -1;
+		
+		total += nwritten;
+		
+		/* terminate the headers */
+		if ((nwritten = g_mime_stream_write_string (stream, newline)) == -1)
+			return -1;
+		
+		total += nwritten;
 	}
 	
-	/* write the content headers */
-	if ((nwritten = g_mime_header_list_write_to_stream (object->headers, stream)) == -1)
-		return -1;
-	
-	total += nwritten;
-	
 	/* write the preface */
 	if (multipart->preface) {
-		/* terminate the headers */
-		if (g_mime_stream_write (stream, "\n", 1) == -1)
+		if ((nwritten = g_mime_stream_write_string (stream, multipart->preface)) == -1)
 			return -1;
 		
-		total++;
+		total += nwritten;
 		
-		if ((nwritten = g_mime_stream_write_string (stream, multipart->preface)) == -1)
+		if ((nwritten = g_mime_stream_write_string (stream, newline)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -194,21 +190,28 @@ multipart_write_to_stream (GMimeObject *object, GMimeStream *stream)
 		part = multipart->children->pdata[i];
 		
 		/* write the boundary */
-		if ((nwritten = g_mime_stream_printf (stream, "\n--%s\n", boundary)) == -1)
+		if ((nwritten = g_mime_stream_printf (stream, "--%s%s", boundary, newline)) == -1)
 			return -1;
 		
 		total += nwritten;
 		
 		/* write this part out */
-		if ((nwritten = g_mime_object_write_to_stream (part, stream)) == -1)
+		if ((nwritten = g_mime_object_write_to_stream (part, options, stream)) == -1)
 			return -1;
 		
 		total += nwritten;
+
+		if (!GMIME_IS_MULTIPART (part) || ((GMimeMultipart *) part)->write_end_boundary) {
+			if ((nwritten = g_mime_stream_write_string (stream, newline)) == -1)
+				return -1;
+			
+			total += nwritten;
+		}
 	}
 	
 	/* write the end-boundary (but only if a boundary is set) */
-	if (boundary) {
-		if ((nwritten = g_mime_stream_printf (stream, "\n--%s--\n", boundary)) == -1)
+	if (multipart->write_end_boundary && boundary) {
+		if ((nwritten = g_mime_stream_printf (stream, "--%s--%s", boundary, newline)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -251,16 +254,7 @@ multipart_encode (GMimeObject *object, GMimeEncodingConstraint constraint)
 GMimeMultipart *
 g_mime_multipart_new (void)
 {
-	GMimeContentType *content_type;
-	GMimeMultipart *multipart;
-	
-	multipart = g_object_newv (GMIME_TYPE_MULTIPART, 0, NULL);
-	
-	content_type = g_mime_content_type_new ("multipart", "mixed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
-	g_object_unref (content_type);
-	
-	return multipart;
+	return g_mime_multipart_new_with_subtype ("mixed");
 }
 
 
@@ -280,12 +274,14 @@ g_mime_multipart_new_with_subtype (const char *subtype)
 	GMimeContentType *content_type;
 	GMimeMultipart *multipart;
 	
-	multipart = g_object_newv (GMIME_TYPE_MULTIPART, 0, NULL);
+	multipart = g_object_new (GMIME_TYPE_MULTIPART, NULL);
 	
 	content_type = g_mime_content_type_new ("multipart", subtype ? subtype : "mixed");
-	g_mime_object_set_content_type (GMIME_OBJECT (multipart), content_type);
+	g_mime_object_set_content_type ((GMimeObject *) multipart, content_type);
 	g_object_unref (content_type);
 	
+	g_mime_multipart_set_boundary (multipart, NULL);
+	
 	return multipart;
 }
 
@@ -398,7 +394,7 @@ multipart_add (GMimeMultipart *multipart, GMimeObject *part)
  * @multipart: a #GMimeMultipart object
  * @part: a #GMimeObject
  *
- * Adds a mime part to the multipart.
+ * Appends a mime part to @multipart.
  **/
 void
 g_mime_multipart_add (GMimeMultipart *multipart, GMimeObject *part)
@@ -443,11 +439,10 @@ multipart_insert (GMimeMultipart *multipart, int index, GMimeObject *part)
 /**
  * g_mime_multipart_insert:
  * @multipart: a #GMimeMultipart object
- * @part: mime part
- * @index: position to insert the mime part
+ * @part: a #GMimeObject
+ * @index: the 0-based index to insert the part
  *
- * Inserts the specified mime part into the multipart at the position
- * @index.
+ * Inserts @part into @multipart at the specified @index.
  **/
 void
 g_mime_multipart_insert (GMimeMultipart *multipart, int index, GMimeObject *part)
@@ -475,9 +470,9 @@ multipart_remove (GMimeMultipart *multipart, GMimeObject *part)
 /**
  * g_mime_multipart_remove:
  * @multipart: a #GMimeMultipart object
- * @part: mime part
+ * @part: a #GMimeObject
  *
- * Removes the specified mime part from the multipart.
+ * Removes the specified @part from @multipart.
  *
  * Returns: %TRUE if the part was removed or %FALSE otherwise.
  **/
@@ -510,9 +505,9 @@ multipart_remove_at (GMimeMultipart *multipart, int index)
 /**
  * g_mime_multipart_remove_at:
  * @multipart: a #GMimeMultipart object
- * @index: position of the mime part to remove
+ * @index: the 0-based index of the part to remove
  *
- * Removes the mime part at position @index from the multipart.
+ * Removes the part at the specified @index from @multipart.
  *
  * Returns: (transfer full): the mime part that was removed or %NULL
  * if the part was not contained within the multipart.
@@ -530,13 +525,13 @@ g_mime_multipart_remove_at (GMimeMultipart *multipart, int index)
 /**
  * g_mime_multipart_replace:
  * @multipart: a #GMimeMultipart object
- * @index: position of the mime part to replace
+ * @index: the 0-based index of the part to replace
  * @replacement: a #GMimeObject to use as the replacement
  *
- * Replaces the mime part at position @index within @multipart with
+ * Replaces the part at the specified @index within @multipart with
  * @replacement.
  *
- * Returns: (transfer full): the mime part that was replaced or %NULL
+ * Returns: (transfer full): the part that was replaced or %NULL
  * if the part was not contained within the multipart.
  **/
 GMimeObject *
@@ -576,11 +571,11 @@ multipart_get_part (GMimeMultipart *multipart, int index)
 /**
  * g_mime_multipart_get_part:
  * @multipart: a #GMimeMultipart object
- * @index: position of the mime part
+ * @index: the 0-based index of the part
  *
- * Gets the mime part at position @index within the multipart.
+ * Gets the part at the specified @index within the multipart.
  *
- * Returns: (transfer none): the mime part at position @index.
+ * Returns: (transfer none): the part at position @index.
  **/
 GMimeObject *
 g_mime_multipart_get_part (GMimeMultipart *multipart, int index)
@@ -609,7 +604,7 @@ multipart_contains (GMimeMultipart *multipart, GMimeObject *part)
 /**
  * g_mime_multipart_contains:
  * @multipart: a #GMimeMultipart object
- * @part: mime part
+ * @part: a #GMimeObject
  *
  * Checks if @part is contained within @multipart.
  *
@@ -643,11 +638,11 @@ multipart_index_of (GMimeMultipart *multipart, GMimeObject *part)
 /**
  * g_mime_multipart_index_of:
  * @multipart: a #GMimeMultipart object
- * @part: mime part
+ * @part: a #GMimeObject
  *
- * Gets the index of @part within @multipart.
+ * Gets the 0-based index of @part within @multipart.
  *
- * Returns: the index of @part within @multipart or %-1 if not found.
+ * Returns: the 0-based index of @part within @multipart or %-1 if not found.
  **/
 int
 g_mime_multipart_index_of (GMimeMultipart *multipart, GMimeObject *part)
@@ -670,9 +665,9 @@ multipart_get_count (GMimeMultipart *multipart)
  * g_mime_multipart_get_count:
  * @multipart: a #GMimeMultipart object
  *
- * Gets the number of mime parts contained within the multipart.
+ * Gets the number of parts contained within @multipart.
  *
- * Returns: the number of mime parts contained within the multipart.
+ * Returns: the number of parts contained within @multipart.
  **/
 int
 g_mime_multipart_get_count (GMimeMultipart *multipart)
@@ -683,41 +678,6 @@ g_mime_multipart_get_count (GMimeMultipart *multipart)
 }
 
 
-static void
-read_random_pool (unsigned char *buffer, size_t bytes)
-{
-#ifdef __unix__
-	size_t nread = 0;
-	ssize_t n;
-	int fd;
-	
-	if ((fd = open ("/dev/urandom", O_RDONLY)) == -1) {
-		if ((fd = open ("/dev/random", O_RDONLY)) == -1)
-			return;
-	}
-	
-	do {
-		do {
-			n = read (fd, (char *) buffer + nread, bytes - nread);
-		} while (n == -1 && errno == EINTR);
-		
-		if (n == -1 || n == 0)
-			break;
-		
-		nread += n;
-	} while (nread < bytes);
-	
-	close (fd);
-#else
-	size_t i;
-	
-	srand ((unsigned int) time (NULL));
-	
-	for (i = 0; i < bytes; i++)
-		buffer[i] = (unsigned char) (rand () % 256);
-#endif
-}
-
 static void
 multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 {
@@ -729,7 +689,7 @@ multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 		guint32 save = 0;
 		int state = 0;
 		
-		read_random_pool (digest, 16);
+		g_mime_read_random_pool (digest, 16);
 		
 		strcpy (bbuf, "=-");
 		p = (unsigned char *) bbuf + 2;
@@ -739,7 +699,7 @@ multipart_set_boundary (GMimeMultipart *multipart, const char *boundary)
 		boundary = bbuf;
 	}
 	
-	g_mime_object_set_content_type_parameter (GMIME_OBJECT (multipart), "boundary", boundary);
+	g_mime_object_set_content_type_parameter ((GMimeObject *) multipart, "boundary", boundary);
 }
 
 
