@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2013 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -56,6 +56,9 @@ struct _GMimeMultipart {
 	char *boundary;
 	char *preface;
 	char *postface;
+	
+	/* < private > */
+	gboolean write_end_boundary;
 };
 
 struct _GMimeMultipartClass {
