@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -129,7 +129,7 @@ test_stream_gets (GMimeStream *stream, const char *filename)
 	char sbuf[100], rbuf[100];
 	ssize_t slen;
 	FILE *fp;
-
+	
 	/* '0x1a' character is treated as EOF (Ctrl+Z) on Windows if file is opened in text mode,
 	 *  thus it's opened in binary mode.
 	 */
@@ -168,27 +168,14 @@ test_stream_buffer_gets (const char *filename)
 	
 	stream = g_mime_stream_fs_new (fd);
 	
-	testsuite_check ("GMimeStreamBuffer::block gets");
+	testsuite_check ("GMimeStreamBuffer::gets()");
 	try {
 		g_mime_stream_reset (stream);
 		buffered = g_mime_stream_buffer_new (stream, GMIME_STREAM_BUFFER_BLOCK_READ);
 		test_stream_gets (buffered, filename);
 		testsuite_check_passed ();
 	} catch (ex) {
-		testsuite_check_failed ("GMimeStreamBuffer::block gets() failed: %s",
-					ex->message);
-	} finally {
-		g_object_unref (buffered);
-	}
-	
-	testsuite_check ("GMimeStreamBuffer::cache gets");
-	try {
-		g_mime_stream_reset (stream);
-		buffered = g_mime_stream_buffer_new (stream, GMIME_STREAM_BUFFER_CACHE_READ);
-		test_stream_gets (buffered, filename);
-		testsuite_check_passed ();
-	} catch (ex) {
-		testsuite_check_failed ("GMimeStreamBuffer::block gets() failed: %s",
+		testsuite_check_failed ("GMimeStreamBuffer::gets() failed: %s",
 					ex->message);
 	} finally {
 		g_object_unref (buffered);
@@ -240,8 +227,6 @@ test_stream_mem (const char *filename)
 #endif
 
 
-
-
 static gboolean
 check_stream_fs (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
 {
@@ -344,7 +329,7 @@ check_stream_mmap (const char *input, const char *output, const char *filename,
 #endif /* HAVE_MMAP */
 
 static gboolean
-check_stream_buffer_block (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
+check_stream_buffer (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
 {
 	GMimeStream *streams[2], *stream;
 	Exception *ex = NULL;
@@ -378,41 +363,6 @@ check_stream_buffer_block (const char *input, const char *output, const char *fi
 	return TRUE;
 }
 
-static gboolean
-check_stream_buffer_cache (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
-{
-	GMimeStream *streams[2], *stream;
-	Exception *ex = NULL;
-	int fd[2];
-	
-	if ((fd[0] = open (input, O_RDONLY, 0)) == -1)
-		return FALSE;
-	
-	if ((fd[1] = open (output, O_RDONLY, 0)) == -1) {
-		close (fd[0]);
-		return FALSE;
-	}
-	
-	streams[0] = g_mime_stream_fs_new (fd[0]);
-	stream = g_mime_stream_buffer_new (streams[0], GMIME_STREAM_BUFFER_CACHE_READ);
-	g_object_unref (streams[0]);
-	streams[0] = g_mime_stream_substream (stream, start, end);
-	g_object_unref (stream);
-	
-	streams[1] = g_mime_stream_fs_new (fd[1]);
-	
-	if (!streams_match (streams, filename))
-		ex = exception_new ("GMimeStreamBuffer (Cache Mode) streams did not match for `%s'", filename);
-	
-	g_object_unref (streams[0]);
-	g_object_unref (streams[1]);
-	
-	if (ex != NULL)
-		throw (ex);
-	
-	return TRUE;
-}
-
 static gboolean
 check_stream_gio (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
 {
@@ -433,7 +383,7 @@ check_stream_gio (const char *input, const char *output, const char *filename, g
 	streams[1] = g_mime_stream_fs_new (fd);
 	
 	if (!streams_match (streams, filename))
-		ex = exception_new ("GMimeStreamBuffer (Cache Mode) streams did not match for `%s'", filename);
+		ex = exception_new ("GMimeStreamGIO streams did not match for `%s'", filename);
 	
 	g_object_unref (streams[0]);
 	g_object_unref (streams[1]);
@@ -451,14 +401,13 @@ static struct {
 	const char *what;
 	checkFunc check;
 } checks[] = {
-	{ "GMimeStreamFs",                  check_stream_fs           },
-	{ "GMimeStreamFile",                check_stream_file         },
+	{ "GMimeStreamFs",     check_stream_fs     },
+	{ "GMimeStreamFile",   check_stream_file   },
 #ifdef HAVE_MMAP
-	{ "GMimeStreamMmap",                check_stream_mmap         },
+	{ "GMimeStreamMmap",   check_stream_mmap   },
 #endif /* HAVE_MMAP */
-	{ "GMimeStreamBuffer (block mode)", check_stream_buffer_block },
-	{ "GMimeStreamBuffer (cache mode)", check_stream_buffer_cache },
-	{ "GMimeStreamGIO",                 check_stream_gio          },
+	{ "GMimeStreamBuffer", check_stream_buffer },
+	{ "GMimeStreamGIO",    check_stream_gio    },
 };
 
 static void
@@ -597,7 +546,6 @@ gen_test_data (const char *datadir, char **stream_name)
 		start = (gint64) (size * (rand () / (RAND_MAX + 1.0)));
 		len = (gint64) (size * (rand () / (RAND_MAX + 1.0)));
 		if (start + len > size) {
-			len = size - start;
 			end = -1;
 		} else {
 			end = start + len;
@@ -629,7 +577,7 @@ int main (int argc, char **argv)
 	const char *dent;
 	int i;
 	
-	g_mime_init (0);
+	g_mime_init ();
 	
 	testsuite_init (argc, argv);
 	
